@@ -1,12 +1,13 @@
+using System;
 
 namespace FSpot {
 	public class PhotoImageView : ImageView {
-		public static double ZoomMultipler = 1.1;
-
 		public delegate void PhotoChangedHandler (PhotoImageView view);
 		public event PhotoChangedHandler PhotoChanged;
 		
 		protected BrowsablePointer item;
+		protected FSpot.Loupe loupe;
+		protected FSpot.Loupe sharpener;
 		
 		public PhotoImageView (IBrowsableCollection query)
 		{
@@ -14,13 +15,12 @@ namespace FSpot {
 			loader.AreaUpdated += HandlePixbufAreaUpdated;
 			loader.AreaPrepared += HandlePixbufPrepared;
 			loader.Done += HandleDone;
-
+			
 			this.SizeAllocated += HandleSizeAllocated;
 			this.KeyPressEvent += HandleKeyPressEvent;
 			this.ScrollEvent += HandleScrollEvent;
-			query.ItemChanged += HandleQueryItemChanged;
 			this.item = new BrowsablePointer (query, -1);
-			item.IndexChanged += PhotoIndexChanged;
+			item.Changed += PhotoItemChanged;
 			this.Destroyed += HandleDestroyed;
 			this.SetTransparentColor (this.Style.BaseColors [(int)Gtk.StateType.Normal]);
 		}
@@ -45,12 +45,12 @@ namespace FSpot {
 			set {
 				if (query != null) {
 					query.Changed -= HandleQueryChanged;
-					query.ItemChanged -= HandleQueryItemChanged;
+					query.ItemsChanged -= HandleQueryItemsChanged;
 				}
 
 				query = value;
 				query.Changed += HandleQueryChanged;
-				query.ItemChanged += HandleQueryItemChanged;
+				query.ItemsChanged += HandleQueryItemsChanged;
 			}
 #endif
 		}
@@ -66,21 +66,7 @@ namespace FSpot {
 			if (!Item.IsValid)
 				return;
 			
-			PhotoIndexChanged (Item, null);
-			
-		}
-		/*
-		private void HandleQueryChanged (IBrowsableCollection browsable)
-		{
-			if (query == browsable)
-				Reload ();
-		}
-		*/
-
-		public void HandleQueryItemChanged (IBrowsableCollection browsable, int item)
-		{
-			if (item == Item.Index)
-				Reload ();
+			PhotoItemChanged (Item, null);
 		}
 
 		// Display.
@@ -104,6 +90,7 @@ namespace FSpot {
 #if true
 			System.Uri uri = Item.Current.DefaultVersionUri;
 			try {
+
 				Gdk.Pixbuf thumb = new Gdk.Pixbuf (ThumbnailGenerator.ThumbnailPath (uri));
 				if (thumb != null && next != null)
 					thumb.Composite (next, 0, 0,
@@ -129,6 +116,7 @@ namespace FSpot {
 			if (prev != null)
 				prev.Dispose ();
 
+			UpdateMinZoom ();
 			this.ZoomFit ();
 		}
 
@@ -159,27 +147,31 @@ namespace FSpot {
 								      PixbufUtils.ErrorPixbuf.Width, 
 								      PixbufUtils.ErrorPixbuf.Height);
 
+				UpdateMinZoom ();
 				this.ZoomFit ();
 			} else {
 				this.Pixbuf = loader.Pixbuf;
 
-				if (!loader.Prepared)
+				if (!loader.Prepared) {
+					UpdateMinZoom ();
 					this.ZoomFit ();
+				}
 			}
 
 			if (prev != this.Pixbuf && prev != null)
 				prev.Dispose ();
-		}		
+		}
 		
 		private bool fit = true;
 		public bool Fit {
 			get {
-				return fit;
+				return (Zoom == MIN_ZOOM);
 			}
 			set {
-				fit = value;
-				if (fit)
+				if (!fit && value)
 					ZoomFit ();
+				
+				fit = value;
 			}
 		}
 
@@ -192,8 +184,33 @@ namespace FSpot {
 			}
 			
 			set {
-				this.Fit = false;
-				this.SetZoom (value, value);
+				//Console.WriteLine ("Setting zoom to {0}, MIN = {1}", value, MIN_ZOOM);
+				value = System.Math.Min (value, MAX_ZOOM);
+				value = System.Math.Max (value, MIN_ZOOM);
+
+				double zoom = Zoom;
+				if (value == zoom)
+					return;
+
+				if (System.Math.Abs (zoom - value) < System.Double.Epsilon)
+					return;
+
+				if (value == MIN_ZOOM)
+					this.Fit = true;
+				else {
+					this.Fit = false;
+					this.SetZoom (value, value);
+				}
+			}
+		}
+		
+		// Zoom scaled between 0.0 and 1.0
+		public double NormalizedZoom {
+			get {
+				return (Zoom - MIN_ZOOM) / (MAX_ZOOM - MIN_ZOOM);
+			}
+			set {
+				Zoom = (value * (MAX_ZOOM - MIN_ZOOM)) + MIN_ZOOM;
 			}
 		}
 		
@@ -201,11 +218,10 @@ namespace FSpot {
 		{
 			if (fit)
 				ZoomFit ();
-		}	
+		}
 
 		bool load_async = true;
 		FSpot.AsyncPixbufLoader loader;
-		FSpot.AsyncPixbufLoader next_loader;
 
 		private void LoadErrorImage (System.Exception e)
 		{
@@ -220,26 +236,33 @@ namespace FSpot {
 			if (old != null)
 				old.Dispose ();
 			
+			UpdateMinZoom ();
 			this.ZoomFit ();
 		}
 
-		private void PhotoIndexChanged (BrowsablePointer item, IBrowsableItem old_item) 
+		private void PhotoItemChanged (BrowsablePointer item, BrowsablePointerChangedArgs args) 
 		{
 			// If it is just the position that changed fall out
-			if (old_item != null && Item.IsValid && this.Item.Current.DefaultVersionUri == old_item.DefaultVersionUri)
+			if (args.PreviousItem != null && 
+			    Item.IsValid && 
+			    this.Item.Current.DefaultVersionUri == args.PreviousItem.DefaultVersionUri)
 				return;
 
 			if (load_async) {
+				Gdk.Pixbuf old = this.Pixbuf;
 				try {
-					if (Item.IsValid)
-						loader.Load (Item.Current.DefaultVersionUri.LocalPath);
-					else
+					if (Item.IsValid) {
+						System.Uri uri = Item.Current.DefaultVersionUri;
+						loader.Load (uri.LocalPath);
+					} else
 						LoadErrorImage (null);
 
 				} catch (System.Exception e) {
 					System.Console.WriteLine (e.ToString ());
 					LoadErrorImage (e);
 				}
+				if (old != null)
+					old.Dispose ();
 			} else {	
 				Gdk.Pixbuf old = this.Pixbuf;
 				this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
@@ -247,6 +270,7 @@ namespace FSpot {
 				if (old != null)
 					old.Dispose ();
 
+				UpdateMinZoom ();
 				this.ZoomFit ();
 			}
 			
@@ -255,7 +279,17 @@ namespace FSpot {
 			if (PhotoChanged != null)
 				PhotoChanged (this);
 		}
-
+		
+		public void ZoomIn ()
+		{
+			Zoom = Zoom * ZOOM_FACTOR;
+		}
+		
+		public void ZoomOut ()
+		{
+			Zoom = Zoom / ZOOM_FACTOR;
+		}
+		
 		private void ZoomFit ()
 		{
 			Gdk.Pixbuf pixbuf = this.Pixbuf;
@@ -271,9 +305,9 @@ namespace FSpot {
 								   (uint) available_height,
 								   (uint) pixbuf.Width, 
 								   (uint) pixbuf.Height, 
-								   false);
+								   true);
 			
-			double image_zoom = zoom_to_fit;
+			double image_zoom = Math.Min(1.0, zoom_to_fit);
 			/*
 			System.Console.WriteLine ("Zoom = {0}, {1}, {2}", image_zoom, 
 						  available_width, 
@@ -288,12 +322,32 @@ namespace FSpot {
 			if (scrolled != null)
 				scrolled.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic);
 		}
-
+		
 		[GLib.ConnectBefore]
 		private void HandleKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 		{
 			// FIXME I really need to figure out why overriding is not working
 			// for any of the default handlers.
+			args.RetVal = true;
+		
+			// Check for KeyPad arrow keys, which scroll the window when zoomed in
+			// but should go to the next/previous photo when not zoomed (no scrollbars)
+			if (this.Fit) {
+				switch (args.Event.Key) {
+				case Gdk.Key.Up:
+				case Gdk.Key.Left:
+				case Gdk.Key.KP_Up:
+				case Gdk.Key.KP_Left:
+					this.Item.MovePrevious ();
+					return;
+				case Gdk.Key.Down:
+				case Gdk.Key.Right:
+				case Gdk.Key.KP_Down:
+				case Gdk.Key.KP_Right:
+					this.Item.MoveNext ();
+					return;
+				}
+			}
 
 			switch (args.Event.Key) {
 			case Gdk.Key.Page_Up:
@@ -301,38 +355,57 @@ namespace FSpot {
 				this.Item.MovePrevious ();
 				break;
 			case Gdk.Key.Home:
+			case Gdk.Key.KP_Home:
 				this.Item.Index = 0;
 				break;
 			case Gdk.Key.End:
+			case Gdk.Key.KP_End:
 				this.Item.Index = this.Query.Count - 1;
 				break;
-			case Gdk.Key.space:
 			case Gdk.Key.Page_Down:
 			case Gdk.Key.KP_Page_Down:
+			case Gdk.Key.space:
+			case Gdk.Key.KP_Space:
 				this.Item.MoveNext ();
 				break;
 			case Gdk.Key.Key_0:
+			case Gdk.Key.KP_0:
 				this.Fit = true;
 				break;
 			case Gdk.Key.Key_1:
+			case Gdk.Key.KP_1:
 				this.Zoom =  1.0;
 				break;
 			case Gdk.Key.Key_2:
+			case Gdk.Key.KP_2:
 				this.Zoom = 2.0;
 				break;
-			case Gdk.Key.KP_Subtract:
 			case Gdk.Key.minus:
-				this.Zoom /= ZoomMultipler;
+			case Gdk.Key.KP_Subtract:
+				ZoomOut ();
 				break;
+			case Gdk.Key.s:
+				if (sharpener == null)
+					sharpener = new Sharpener (this);
+
+				sharpener.Show ();
+				break;
+			case Gdk.Key.v:
+				if (loupe == null)
+					loupe = new Loupe (this);
+
+				loupe.Show ();
+				break;
+			case Gdk.Key.equal:
 			case Gdk.Key.plus:
 			case Gdk.Key.KP_Add:
-				this.Zoom *= ZoomMultipler;
+				ZoomIn ();
 				break;
 			default:
 				args.RetVal = false;
 				return;
 			}
-			args.RetVal = true;
+
 			return;
 		}
 		
