@@ -3,7 +3,7 @@ using System;
 using System.Collections;
 
 namespace FSpot {
-	public class PhotoQuery : FSpot.IPhotoCollection {
+	public class PhotoQuery : FSpot.IBrowsableCollection {
 		private Photo [] photos;
 		private PhotoStore store;
 		private Tag [] tags;
@@ -28,7 +28,7 @@ namespace FSpot {
 
 		// IPhotoCollection Interface
 		public event FSpot.IBrowsableCollectionChangedHandler Changed;
-		public event FSpot.IBrowsableCollectionItemChangedHandler ItemChanged;
+		public event FSpot.IBrowsableCollectionItemsChangedHandler ItemsChanged;
 		
 		public IBrowsableItem this [int index] {
 			get {
@@ -62,6 +62,7 @@ namespace FSpot {
 			set {
 				tags = value;
 				photos = store.Query (tags, range);
+				untagged = false;
 				RequestReload ();
 			}
 		}
@@ -73,9 +74,28 @@ namespace FSpot {
 			set {
 				range = value;
 				photos = store.Query (tags, range);
+				untagged = false;
 				RequestReload ();
 			}
 		}
+		
+		private bool untagged = false;
+		public bool Untagged {
+			get {
+				return untagged;
+			}
+			set {
+				untagged = value;
+
+				if (untagged) {
+					range = null;
+					tags = null;
+
+					photos = store.QueryUntagged ();
+					RequestReload ();
+				}
+			}
+		}
 
 		public void RequestReload ()
 		{
@@ -96,7 +116,7 @@ namespace FSpot {
 		
 		public void MarkChanged (int index)
 		{
-			ItemChanged (this, index);
+			ItemsChanged (this, new BrowsableArgs (index));
 		}
 	}
 }
